package processing

import (
	"cloud.o-forge.io/core/oc-lib/dbs/mongo"
	"cloud.o-forge.io/core/oc-lib/models/utils"
)

type ProcessingMongoAccessor struct {
	utils.AbstractAccessor
}

func (pma *ProcessingMongoAccessor) DeleteOne(id string) (utils.DBObject, error) {
	return pma.GenericDeleteOne(id, pma)
}

func (pma *ProcessingMongoAccessor) UpdateOne(set map[string]interface{}, id string) (utils.DBObject, error) {
	return pma.GenericUpdateOne(set, id, pma)
}

func (pma *ProcessingMongoAccessor) StoreOne(data utils.DBObject) (utils.DBObject, error) {
	err := pma.Validate(data.(*Processing))
	if err != nil {
		return nil, err
	}
	id, err := mongo.MONGOService.StoreOne(data.(*Processing), pma.GetType())
	if err != nil {
		pma.Logger.Error().Msg("Could not store " + data.GetName() + " to db. Error: " + err.Error())
		return nil, err
	}
	return pma.LoadOne(id)
}

func (pma *ProcessingMongoAccessor) LoadOne(id string) (utils.DBObject, error) {

	var processing Processing

	res_mongo, err := mongo.MONGOService.LoadOne(id, pma.GetType())
	if err != nil {
		pma.Logger.Error().Msg("Could not retrieve " + id + " from db. Error: " + err.Error())
		return nil, err
	}

	res_mongo.Decode(&processing)

	return &processing, nil
}
