package storage

import (
	"cloud.o-forge.io/core/oc-lib/dbs/mongo"
	"cloud.o-forge.io/core/oc-lib/models/utils"
)

type StorageMongoAccessor struct {
	utils.AbstractAccessor
}

func (sma *StorageMongoAccessor) DeleteOne(id string) (utils.DBObject, error) {
	return sma.GenericDeleteOne(id, sma)
}

func (sma *StorageMongoAccessor) UpdateOne(set map[string]interface{}, id string) (utils.DBObject, error) {
	return sma.GenericUpdateOne(set, id, sma)
}

func (sma *StorageMongoAccessor) StoreOne(data utils.DBObject) (utils.DBObject, error) {
	err := sma.Validate(data.(*Storage))
	if err != nil {
		return nil, err
	}
	id, err := mongo.MONGOService.StoreOne(data.(*Storage), sma.GetType())
	if err != nil {
		sma.Logger.Error().Msg("Could not store " + data.GetName() + " to db. Error: " + err.Error())
		return nil, err
	}
	return sma.LoadOne(id)
}

func (sma *StorageMongoAccessor) LoadOne(id string) (utils.DBObject, error) {

	var storage Storage

	res_mongo, err := mongo.MONGOService.LoadOne(id, sma.GetType())
	if err != nil {
		sma.Logger.Error().Msg("Could not retrieve " + id + " from db. Error: " + err.Error())
		return nil, err
	}

	res_mongo.Decode(&storage)

	return &storage, nil
}
